#include "notifydbusinterface.h"

#include <QDebug>
#include <QDBusReply>

namespace {
const char *notify_service = "org.freedesktop.Notifications";
const char *notify_path = "/org/freedesktop/Notifications";
const char *notify_interface = "org.freedesktop.Notifications";
}

NotifyDBusInterface::NotifyDBusInterface(QObject *parent)
    : QObject{parent}
    , m_notifyInterface(notify_service, notify_path, notify_interface)
{
    if (!m_notifyInterface.isValid()) {
        qWarning() << "Create dbus interface" << notify_service
                   << notify_path << notify_interface << "failed.";
    }

    initConnections();
}

void NotifyDBusInterface::initConnections()
{
    connect(&m_notifyInterface, SIGNAL(NotificationClosed(uint,uint)),
            this, SLOT(handleNotifyClose(uint,uint)));

    connect(&m_notifyInterface, SIGNAL(ActionInvoked(uint,QString)),
            this, SLOT(handleActionInvoked(uint,QString)));
}

int NotifyDBusInterface::sendNotification(
    const QString &summary, const QString &body, const QStringList &actions, const QVariantMap &hints, uint replace)
{
    qDebug() << "Send system notify:" << body;

    const QString app_name = QObject::tr("Settings");
    const QString app_icon = "ukui-control-center";
    const int expire_timeout = 0;

    QDBusReply<uint> reply =
        m_notifyInterface.call("Notify", app_name, replace, app_icon,
                               summary, body, actions, hints, expire_timeout);
    if (!reply.isValid()) {
        qWarning() << reply.error().name() << reply.error().message();
        return -1;
    }

    return reply.value();
}

void NotifyDBusInterface::setNotifyCloseCallback(NotifyCloseCallback callback)
{
    m_notifyCloseCallbacks.emplace_back(std::move(callback));
}

void NotifyDBusInterface::handleNotifyClose(uint notifyId, uint reason)
{
    for (auto &callback : m_notifyCloseCallbacks) {
        callback(notifyId, reason);
    }
}

void NotifyDBusInterface::setActionInvokedCallback(ActionInvokedCallback callback)
{
    m_actionInvokedCallbacks.emplace_back(std::move(callback));
}

void NotifyDBusInterface::handleActionInvoked(uint notifyId, const QString &actionKey)
{
    for (auto &callback : m_actionInvokedCallbacks) {
        callback(notifyId, actionKey);
    }
}
