#ifndef NOTIFYDBUSINTERFACE_H
#define NOTIFYDBUSINTERFACE_H

#include <QObject>
#include <QDBusInterface>

class NotifyDBusInterface : public QObject
{
    Q_OBJECT
public:
    using NotifyCloseCallback = std::function<void(uint,uint)>;
    using ActionInvokedCallback = std::function<void(uint,QString)>;

    explicit NotifyDBusInterface(QObject *parent = nullptr);

    int sendNotification(const QString &summary, const QString &body, const QStringList &actions,
                         const QVariantMap &hints, uint replace = 0);
    void setNotifyCloseCallback(NotifyCloseCallback callback);
    void setActionInvokedCallback(ActionInvokedCallback callback);

private Q_SLOTS:
    void handleNotifyClose(uint notifyId, uint reason);
    void handleActionInvoked(uint notifyId, const QString &actionKey);

private:
    void initConnections();

    QDBusInterface m_notifyInterface;
    std::vector<NotifyCloseCallback> m_notifyCloseCallbacks;
    std::vector<ActionInvokedCallback> m_actionInvokedCallbacks;
};


#endif // NOTIFYDBUSINTERFACE_H
