#include "schedpolicy.h"
#include <algorithm>

namespace sched_policy {

DeviceMode deviceModeFromString(const std::string &mode) {
    static std::map<std::string, DeviceMode> deviceModeMap {
        {"PC", DeviceMode::PC},
        {"Tablet", DeviceMode::Tablet},
        {"SoftFreeze", DeviceMode::SoftFreeze}
    };

    if (deviceModeMap.find(mode) == deviceModeMap.end()) {
        return DeviceMode::Unknown;
    }

    return deviceModeMap[mode];
}

PowerMode powerModeFromString(const std::string &mode) {
    static std::map<std::string, PowerMode> powerModeMap {
        {"Balance", PowerMode::Balance},
        {"Save", PowerMode::Save},
        {"Performance", PowerMode::Performance}
    };

    if (powerModeMap.find(mode) == powerModeMap.end()) {
        return PowerMode::Unknown;
    }
    return powerModeMap[mode];
}

GroupType groupTypeFromString(const std::string &type) {
    static std::map<std::string, GroupType> groupTypeMap {
        {"SessionScope", GroupType::SessionScopeGroup},
        {"Session", GroupType::SessionGroup},
        {"Top", GroupType::TopGroup},
        {"Focus", GroupType::FocusGroup},
        {"Foreground", GroupType::ForegroundGroup},
        {"Background", GroupType::BackgroundGroup},
        {"Cached", GroupType::CachedGroup},
        {"Default", GroupType::DefaultGroup}
    };

    if (groupTypeMap.find(type) == groupTypeMap.end()) {
        return GroupType::Unknown;
    }

    return groupTypeMap[type];
}

std::string deviceModeToString(DeviceMode mode)
{
    static std::map<DeviceMode, std::string> deviceModeMap {
        {DeviceMode::PC, "PC"},
        {DeviceMode::Tablet, "Tablet"},
        {DeviceMode::SoftFreeze, "SoftFreeze"}
    };

    return deviceModeMap[mode];
}

std::string generatePolicyId(const std::string &deviceMode, const std::string &powerMode) {
    auto toLower = [](const std::string &str) {
        std::string tmp(str);
        std::transform(tmp.begin(), tmp.end(), tmp.begin(), [](unsigned char c){ return std::tolower(c); });
        return tmp;    
    };

    if (deviceModeFromString(deviceMode) == DeviceMode::Unknown ||
        powerModeFromString(powerMode) == PowerMode::Unknown) {
        return std::string();
    }

    return toLower(deviceMode) + "_" + toLower(powerMode);
}

std::string generatePolicyId(DeviceMode deviceMode, PowerMode powerMode) {
    static std::map<PowerMode, std::string> powerModeMap {
        {PowerMode::Balance, "Balance"},
        {PowerMode::Save, "Save"},
        {PowerMode::Performance, "Performance"}
    };

    std::string deviceModeStr = deviceModeToString(deviceMode);

    if (deviceModeStr.empty() ||
        powerModeMap.find(powerMode) == powerModeMap.end()) {
        return std::string();
    }

    return generatePolicyId(deviceModeStr, powerModeMap[powerMode]);
}

} // namespace sched_policy
