#ifndef TIMEWHEEL_H
#define TIMEWHEEL_H

#include <QObject>
#include <functional>
#include <variant>
#include <map>
#include <memory>

class QTimer;
namespace timewheel {

using TimeoutCallback = std::function<void()>;

struct Timer
{
    quint64 timerId;
    int rotation;
    int tickSlot;
    TimeoutCallback callback;
};

class TimeWheel : public QObject
{
public:
    explicit TimeWheel(QObject *parent = nullptr);
    unsigned long long addTimer(int timeout,
                                const TimeoutCallback &callback);

    void deleteTimer(unsigned long long timerId);

private Q_SLOTS:
    void tick();

private:
    // <slotnumber, list<timer>>
    using SlotTimerMap = std::map<int, std::list<std::unique_ptr<Timer>>>;
    QTimer *m_timer;
    int m_currentSlot;
    unsigned long long m_currentTimerCount;

    SlotTimerMap m_slotTimers;
    const int kSlostInterval = 1; // 1s
    const int kSlotNumber = 60;
};

} // namsapce timewheel;


#endif // TIMEWHEEL_H
