#ifndef BATTERYREMAININGTIMEESTIMATOR_H
#define BATTERYREMAININGTIMEESTIMATOR_H

#include <QTimer>
#include <functional>
#include <memory>
#include "core/systemenery/systemenergy.h"
#include "core/consumptionmonitor/processgroupconsumptionmonitor.h"

class BatteryRemainingTimeEstimator
{
public:
    using RemainingTimeChangedCallback = std::function<void(int)>;
    using FrozenProcessGroupIdsGetter = std::function<std::vector<std::string>()>;

    BatteryRemainingTimeEstimator(
        std::shared_ptr<SystemEnergy> systemEnergy,
        std::shared_ptr<ProcessGroupConsumptionMonitor> processGroupConsumptionMonitor,
        int estimateInterval,
        FrozenProcessGroupIdsGetter frozenProcessGroupIdsGetter);

    void setIncreasedRemainTimeChangedCallback(RemainingTimeChangedCallback callback);

    void startEstimate();
    void stopEstimate();

private:
    void estimateRemainingTime();
    double calculateFrozenGroupsEnergyRate(const std::vector<std::string> &groupIds);
    void setIncreasedRemainingTime(int time);

private:
    std::shared_ptr<SystemEnergy> m_systemEnergy;
    std::shared_ptr<ProcessGroupConsumptionMonitor> m_processGroupConsumptionMonitor;
    int m_estimateInterval;
    QTimer m_timer;
    RemainingTimeChangedCallback m_increasedRemainingTimeChangedCallback;
    FrozenProcessGroupIdsGetter m_frozenProcessGroupIdsGetter;
};

#endif // BATTERYREMAININGTIMEESTIMATOR_H
