#ifndef DEVICESTATE_H
#define DEVICESTATE_H

#include <QObject>
#include <QDBusInterface>
#include <QGSettings/qgsettings.h>
#include <memory>
#include "schedpolicy.h"

class PowerManager : public QObject
{
    Q_OBJECT

public:
    PowerManager();

    sched_policy::PowerMode currentPowerMode() const;
    sched_policy::PowerType currentPowerType() const;

Q_SIGNALS:
    void powerModeChanged(sched_policy::PowerMode powerMode);
    void powerTypeChanged(sched_policy::PowerType powerType);

private Q_SLOTS:
    void onPropertiesChanged(
        QString name, QMap<QString, QVariant> value, QStringList);

private:
    void initGSettings();
    void initDbusConnection();
    void initPowerMode();
    void handlePowerModeChanged(int value);

private:
    sched_policy::PowerType m_currentPowerType;
    sched_policy::PowerMode m_currentPowerMode;

    std::unique_ptr<QGSettings> m_gsettings;
};

class DeviceModeManager : public QObject
{
    Q_OBJECT

public:
    DeviceModeManager(bool softFreezeEnabled);

    sched_policy::DeviceMode currentDeviceMode() const;

    void handleSoftFreezeModeEnabledChanged(bool enabled);

Q_SIGNALS:
    void deviceModeChanged(sched_policy::DeviceMode mode);

private Q_SLOTS:
    void updateDeviceMode();

private:
    void resetDeviceMode();
    void initConnections();

    bool isTabletMode();

private:
    bool m_softFreezeModeEnabled;
    sched_policy::DeviceMode m_currentDeviceMode;
};

#endif // DEVICESTATE_H
