#include "cpuenergymeter.h"

CpuEnergyMeter::CpuEnergyMeter()
    : m_startEnergy{0}
    , m_endEnergy{0}
{
}

void CpuEnergyMeter::startMeasurement()
{
    m_startTimePoint = std::chrono::high_resolution_clock::now();
    m_startEnergy = m_dbusInterface.getCpuEnergyConsumption();
}

void CpuEnergyMeter::endMeasurement()
{
    m_endTimePoint = std::chrono::high_resolution_clock::now();
    m_endEnergy = m_dbusInterface.getCpuEnergyConsumption();
}

double CpuEnergyMeter::energyConsumption() const
{
    return m_endEnergy - m_startEnergy;
}

double CpuEnergyMeter::energyConsumptionRate() const
{
    if (m_endTimePoint <= m_startTimePoint) {
        return 0;
    }

    auto deltaTime = std::chrono::duration_cast
        <std::chrono::milliseconds>(m_endTimePoint - m_startTimePoint);

    // 计算功率，转换成瓦
    return energyConsumption() / (deltaTime.count() / 1000);
}
