#ifndef EXCEPTIONGROUPPROCESSWATCHER_H
#define EXCEPTIONGROUPPROCESSWATCHER_H

#include <QObject>
#include <string>
#include <thread>
#include <future>
#include <iostream>
#include <map>
#include <vector>
#include <sstream>
#include <fstream>
#include <unordered_map>
#include <atomic>
#include <condition_variable>
#include "schedpolicy.h"
#include "processinfomanager.h"

class ExceptionGroupProcessWatcher : public QObject
{
    Q_OBJECT
public:
    explicit ExceptionGroupProcessWatcher(
        const std::shared_ptr<ProcessInfoManager> &processInfoManager, QObject *parent = nullptr);
    void addPath(std::string path, sched_policy::GroupType groupType);
    void startWatcher();
    void stopWatcher();

Q_SIGNALS:
    void exceptionSessionAppCaught(const std::string &desktopFile, int pid);

private:
    std::vector<int> readFileToPids(const std::string &file) const;
    std::vector<int> findExceptionPids(const std::vector<int> &oldPids, const std::vector<int> &newPids) const;
    void handleExceptionPids(const std::vector<int> &pids, sched_policy::GroupType groupType);

private:
    struct WatchedGroupInfo {
        std::string path;
        sched_policy::GroupType groupType;
        std::vector<int> pids;
    };

private:
    std::map<std::string, WatchedGroupInfo> m_watchedGroupPaths;
    std::atomic_bool m_watcherRunning;
    std::future<void> m_watcherFuture;
    std::condition_variable m_stopWatcherConditionVar;
    std::mutex m_mutex;
    std::shared_ptr<ProcessInfoManager> m_processInfoManager;
};

#endif // EXCEPTIONGROUPPROCESSWATCHER_H
