#include "groupmanager.h"
#include <QDebug>
#include "resourcemanagerinterface.h"
#include "processinfohelper.h"

void GroupManager::initGroups()
{
    auto groups = m_groupConfigs["AggregateProfiles"];
    auto controllers = m_groupConfigs["ControllerList"];
    auto attributes = m_groupConfigs["Attributes"];

    m_unitCreator.reset(new GroupManagementUnitCreator(
        std::make_shared<ResourceManagerInterface>(), controllers, attributes));
    
    for (Json::Value::ArrayIndex i=0; i<groups.size(); ++i) {
        auto unit = m_unitCreator->createGroupManagementUnit(groups[i]);
        m_groupUnits[unit->groupType()] = std::move(unit);
    }
}

void GroupManager::excutePolicy(const std::string &policyId)
{
    for (const auto &cgroupUnit : m_groupUnits) {
        cgroupUnit.second->executeResourceLimit(policyId);
    }
}

void GroupManager::setGroupConfigs(const Json::Value &configs)
{
    m_groupConfigs = configs;
    m_resourceManagerInterface.setCurrentUserServiceUnitPropertyEnabled("CPUAccounting", true);
    initGroups();
}

void GroupManager::createProcessGroup(
    const std::string &cgroupName, const std::vector<int> &pids,
    sched_policy::GroupType parentGroupType)
{
    if (m_groupUnits.find(parentGroupType) == m_groupUnits.end()) {
        qWarning() << QString("Create process %1 group but not loaded the parent group info %2.")
                          .arg(QString::fromStdString(cgroupName))
                          .arg((int)parentGroupType);
        return;
    }
    m_groupUnits[parentGroupType]->createTransientProcessGroup(cgroupName, pids);
}

void GroupManager::moveProcessToGroup(
    int parentPid, std::vector<int> &childPids, sched_policy::GroupType parentGroupType)
{
    if (m_groupUnits.find(parentGroupType) == m_groupUnits.end()) {
        qWarning() << QString("Move process to group but not loaded the parent group info %2.")
                          .arg((int)parentGroupType);
        return;
    }
    m_groupUnits[parentGroupType]->moveProcessToGroup(ProcessInfoHelper::cgroup(parentPid), childPids);
}

std::string GroupManager::groupPath(sched_policy::GroupType type)
{
    if (m_groupUnits.find(type) == m_groupUnits.end()) {
        return std::string();
    }

    return m_groupUnits[type]->groupPath();
}

void GroupManager::reclaimProcessGroups(const std::vector<std::string>& groupNames)
{
    m_resourceManagerInterface.reclaimProcessGroups(groupNames);
}
