#include "multimediacontroller.h"
#include <QApplication>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QDBusPendingReply>
#include <QDBusReply>
#include <QDebug>

namespace {

const char *mpris_dbus_path = "/org/mpris/MediaPlayer2";
const char *mpris_dbus_interface = "org.mpris.MediaPlayer2.Player";

}

void MultimediaController::pause(const std::string &serviceName)
{
    if (isPause(serviceName)) {
        return;
    }

    QDBusMessage message = QDBusMessage::createMethodCall(
        QString::fromStdString(serviceName), mpris_dbus_path, mpris_dbus_interface,
        QStringLiteral("PlayPause"));

    QDBusPendingCall pcall = QDBusConnection::sessionBus().asyncCall(message);
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pcall);
    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, qApp,
                     [watcher, serviceName](QDBusPendingCallWatcher *call) {
                         QDBusPendingReply<> reply = *call;
                         if (reply.isError()) {
                             qWarning() << QString("Call mpris dbus %1 error: ")
                                               .arg(QString::fromStdString(serviceName));
                         }
                         call->deleteLater();
                         watcher->deleteLater();
                     });
}

bool MultimediaController::isPause(const std::string &serviceName)
{
    if (!QDBusConnection::sessionBus().isConnected()) {
        qWarning() << "Cannot connect to the D-Bus session bus.";
        return true;
    }

    QDBusInterface mprisDbus(
        QString::fromStdString(serviceName),
        mpris_dbus_path, mpris_dbus_interface,
        QDBusConnection::sessionBus());

    if (!mprisDbus.isValid()) {
        qWarning() << QString("mpris dbus service %1 is not valid.")
                          .arg(QString::fromStdString(serviceName));
        return true;
    }

    QDBusReply<int> reply = mprisDbus.call("PlayState");
    return reply <= 0;
}
