#ifndef PROCESSINFOMANAGER_H
#define PROCESSINFOMANAGER_H

#include "appinfomanager.h"
#include "configmanager.h"

class ProcessInfoManager
{
public:
    using AppChangedCallback = AppInfoManager::AppChangedCallback;

    ProcessInfoManager(const std::shared_ptr<ConfigManager> &configManager);

    void handleWindowAdded(uint wid);
    void handleWindowRemoved(uint wid);
    void handleActiveWindowChanged(uint wid);
    void handleWindowMinimized(uint wid);
    void handleExceptionGroupSessionApp(const std::string &desktopFile, int pid);
    void handleMprisDbusAdded(int pid, const std::string &service);

    void createAppInstance(const std::string &desktopFile, int pid);
    void setAppGroupName(const std::string &appId, const std::string &groupName);

    void forceChangCachedStateTo(sched_policy::AppState state);
    bool forceChangeAppStateByPid(int pid, sched_policy::AppState state);

    void setBackgroundToCachedStateInterval(int interval);

    sched_policy::AppType getAppTypeByPid(int pid);
    std::string syncGetDesktopFileByPid(int pid);

    std::vector<AppInfo> getAllAppInfos();
    std::vector<AppInfo> getAppInfosWithState(sched_policy::AppState appState);

    AppInfo getLatestAppInfoByDesktopFile(
        const std::string &desktopFile, const std::vector<std::string> &args);
    AppInfo getLatestAppInfoByCmdline(const std::string &cmdline);
    AppInfo getAppInfo(const std::string &appId);

    std::string getDesktopFileByPid(int pid);

    void setAppStartedCallback(AppChangedCallback callback);
    void setAppStateChangedCallback(AppChangedCallback callback);
    void setAppChildPidsUpdatedCallback(AppChangedCallback callback);

private:
    AppInfoManager m_appInfoManager;
};

#endif // PROCESSINFOMANAGER_H
