#include "resourcemanagerinterface.h"
#include "misc.h"

void ResourceManagerInterface::createPersistentProcessGroup(const std::string &path, const std::vector<std::string> &cgroupControllers)
{
    m_daemonDbus.createProcessGroup(path, cgroupControllers, std::vector<int>{});
}

void ResourceManagerInterface::createTransientProcessGroup(
    const std::string &path, const std::vector<int> &pids,
    const std::vector<std::string> &cgroupControllers)
{
    m_daemonDbus.createProcessGroup(path, cgroupControllers, pids);
}

void ResourceManagerInterface::setProcessGroupResourceLimit(
    const std::string &path, const std::string &controller,
    const std::string &file, const std::string &value)
{
    m_daemonDbus.setProcessGroupResourceLimit(path, controller, file, value);
}

void ResourceManagerInterface::moveProcessToGroup(
    const std::string &path, const std::vector<std::string> &controllers,
    const std::vector<int> &pids)
{
    m_daemonDbus.moveProcessToGroup(path, controllers, pids);
}

void ResourceManagerInterface::setCurrentUserServiceUnitPropertyEnabled(
    const std::string &property, bool enabled)
{
    const auto &currentUserService = m_systemdDbus.currentUserServiceName();
    m_daemonDbus.setSystemdUnitPropertyEnabled(currentUserService, property, enabled);
}

void ResourceManagerInterface::reclaimProcessGroups(const std::vector<std::string> &groupNames)
{
    m_daemonDbus.reclaimProcessGroups(groupNames);
}

std::string ResourceManagerInterface::getGroupNameFromPath(const std::string &path) const
{
    auto stringList = misc::string::split(path, '/');
    if (stringList.empty()) {
        return std::string();
    }

    return stringList.back();
}

std::string ResourceManagerInterface::getGroupParentSliceFromPath(const std::string &path) const
{
    auto stringList = misc::string::split(path, '/');
    if (stringList.size() < 2) {
        return std::string();
    }

    return stringList.at(stringList.size() - 2);
}
