#include "upowersystemenery.h"
#include <QDBusReply>
#include <QDebug>

namespace {

const char *upower_dbus_service = "org.freedesktop.UPower";
const char *upower_dbus_path = "/org/freedesktop/UPower/devices/DisplayDevice";
const char *upower_dbus_interface = "org.freedesktop.UPower.Device";
const char *upower_dbus_properties_interface = "org.freedesktop.DBus.Properties";

}
UPowerSystemEnery::UPowerSystemEnery()
    : m_upowerDbusInterface(
        upower_dbus_service, upower_dbus_path,
        upower_dbus_properties_interface, QDBusConnection::systemBus())
{
}

double UPowerSystemEnery::energy()
{
    // 瓦时->焦耳
    return getUpowerProperty("Energy").toDouble() * 3600;
}

double UPowerSystemEnery::dischargeRate()
{
    return getUpowerProperty("EnergyRate").toDouble();
}

int UPowerSystemEnery::timeToEmpty()
{
    return getUpowerProperty("TimeToEmpty").toInt();
}

QVariant UPowerSystemEnery::getUpowerProperty(const QString &propertyName)
{
    QDBusReply<QVariant> reply = m_upowerDbusInterface.call("Get", upower_dbus_interface, propertyName);
    if (!reply.isValid()) {
        qWarning() << "Get upower property failed: " << reply.error();
        return QVariant();
    }
    return reply.value();
}
