#include "systemnotifiessender.h"

#include <QDebug>

namespace {
const char *enable_reousrce_limit_key = "0";
}

SystemNotifiesSender::SystemNotifiesSender(std::shared_ptr<ConfigManager> configManager)
    : m_configManager(configManager)
    , m_notifyDBusInterface(new NotifyDBusInterface)
    , m_notifyTimer(new QTimer)
    , m_resourceWarningNotifyId(-1)
{
    m_notifyTimer->setSingleShot(true);
    initConnections();
}

void SystemNotifiesSender::initConnections()
{
    m_notifyDBusInterface->setActionInvokedCallback([this](uint notifyId, const QString &actionKey) {
        handleResourceWarningActionInvoked(notifyId, actionKey.toStdString());
    });
    m_notifyDBusInterface->setNotifyCloseCallback([this](uint notifyId, uint reason) {
        handleResourceWarningNotifyClose(notifyId, reason);
    });
}

void SystemNotifiesSender::handleResourceWarningNotifyClose(uint notifyId, uint reason)
{
    if (notifyId != m_resourceWarningNotifyId)
        return;
    qDebug() << "Receive NotifyClose signal from notify:" << notifyId << reason;

    m_resourceWarningNotifyId = -1;
    m_notifyTimer->start(6000); // after close notify, no more repeated notify within 60s.
}

void SystemNotifiesSender::handleResourceWarningActionInvoked(uint notifyId, const std::string &actionKey)
{
    if (notifyId != m_resourceWarningNotifyId)
        return;
    qDebug() << "Receive ActionInvoked signal from notify:" << notifyId << actionKey.c_str();

    if (actionKey == enable_reousrce_limit_key) {
        m_configManager->setReousrceLimitEnabled(true);
        m_resourceWarningNotifyId = -1;
    }
}

void SystemNotifiesSender::sendResourceWarningNotify()
{
    if (m_resourceWarningNotifyId != -1)
        return;
    if (m_notifyTimer->isActive()) {
        qDebug() << "Just close notify," << m_notifyTimer->remainingTime() << "after repeate.";
        return;
    }

    const QString summary;
    const QString body =
        QObject::tr("The current system is detected to be stuck, we recommend that you enable the hierarchical freezing function");
    const QStringList actions = {enable_reousrce_limit_key, QObject::tr("Open")};
    const QVariantMap hints = {{QString("urgency"), QVariant::fromValue(QString("2"))},
                               {QString("x-ukui-popup-timeout"), QVariant::fromValue(-1)}};

    m_resourceWarningNotifyId = m_notifyDBusInterface->sendNotification(summary, body, actions, hints);
}
