#ifndef CGROUPMANAGER_H
#define CGROUPMANAGER_H

#include <QList>
#include <QString>
#include <memory>
#include <functional>
#include <libcgroup.h>
#include "cgroupv2releasenotification.h"

class CGroupManager
{
public:
    bool createProcessGroup(const QString &groupPath, 
        const QStringList &controllers, const QList<int> &pids);

    bool moveProcessToGroup(const QString &groupPath, 
        const QStringList &controllers, const QList<int> &pids);
        
    bool setProcessGroupResourceLimit(
        const QString &groupPath, const QString &controllerName, 
        const QString &attrFile, const QString &value);

    void removeProcessCGroup(const QString &groupPath);

    QList<int> getGroupPids(const QString &groupName) const;

private:
    using CgroupUniquePtr = std::unique_ptr<cgroup, std::function<void(cgroup *)>>;

    CgroupUniquePtr initCgroupWithPath(const QString &groupPath);
    CgroupUniquePtr createCgroup(const QString &cgroupName, const QStringList &controllers);
    void initControllers(
        cgroup *cgroup, const QStringList &controllers, 
        const QString &notifyOnReleaseValue = {});
    void setCGroupV1ReleaseAgent(const QStringList &controllers);

    void writePidToProcsFile(int pid, const QString &procsFile);

private:
    CGroupV2ReleaseNotification m_cgroupv2ReleaseNotification;
};

#endif // CGROUPMANAGER_H
