#ifndef CGROUPV2RELEASENOTIFICATION_H
#define CGROUPV2RELEASENOTIFICATION_H

#include <sys/epoll.h>
#include <sys/inotify.h>
#include <string>
#include <future>
#include <functional>
#include <map>

class CGroupV2ReleaseNotification
{
public:
    using ReleaseHandler = std::function<void(std::string groupPath)>;

    CGroupV2ReleaseNotification();
    void addWatchGroup(const std::string &groupPath, ReleaseHandler releaseHandler);

private:
    void startWatch();
    int initEpoll();
    void doStartWatch();
    void handleEpollEvents(epoll_event *events, int eventNumber);
    void handleInotifyEvent();
    void handleCGroupReleaseEvent(inotify_event *event);

private:
    int m_inotifyFd;
    int m_epollFd;
    std::future<void> m_watcherFuture;
    std::mutex m_mutex;
    // <watched fd<watched path, release handler>>
    std::map<int, std::pair<std::string, ReleaseHandler>> m_watchedPathInfo;
};

#endif // CGROUPV2RELEASENOTIFICATION_H
