#include <QCoreApplication>
#include <QCommandLineParser>
#include "processmanagerservice.h"
#include "processmanager.h"

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    ProcessManager processManager;
    QCommandLineParser parser;
    parser.setApplicationDescription("Kylin Process Manager Daemon");
    parser.addHelpOption();
    parser.addVersionOption();
    QCommandLineOption removeOption(QStringList() << "r" << "remove", "Remove specified <cgroup path>.", "cgroup path");
    parser.addOption(removeOption);
    parser.process(a);
    if (parser.isSet(removeOption)) {
        QString groupPath = parser.value(removeOption);
        if (groupPath.isEmpty()) {
            qCritical() << "Error: No cgroup path specified.";
            return 1;
        }
        processManager.RemoveProcessCGroup(groupPath);
        return 0;
    }
    
    QDBusConnection connection = QDBusConnection::systemBus();
    if (!connection.registerService("com.kylin.ProcessManagerDaemon") ||
            !connection.registerObject("/com/kylin/ProcessManagerDaemon", &processManager)) {
        qWarning() << "register dbus service failed" << connection.lastError();
        return 0;
    }

    ProcessManagerService processManagerService(&processManager);

    return a.exec();
}
