#include <QApplication>
#include <QTranslator>
#include <ukui-log4qt.h>
#include "processmanager.h"
#include "processmanagerservice.h"

void setWaylandPlatformEnvValue() {
    qputenv("QT_QPA_PLATFORM", "wayland");
}

bool isWaylandPlatform() {
    QByteArray xdgSessionTypeValue = qgetenv("XDG_SESSION_TYPE");
    return xdgSessionTypeValue.compare("wayland", Qt::CaseInsensitive) == 0;
}

int main(int argc, char *argv[])
{
    // 在Xwayland或者wayland平台上以wayland方式启动kylin-process-manager进程
    // 可兼容x与wayland窗口处理事件
    if (isWaylandPlatform()) {
        setWaylandPlatformEnvValue();
    }

    QApplication a(argc, argv);

#ifndef QT_DEBUG
    initUkuiLog4qt(a.applicationName());
#endif
    QTranslator translator;
    translator.load(QLocale::system().name(), "/usr/share/kylin-process-manager/");
    a.installTranslator(&translator);

    ProcessManager processManager;
    ProcessManagerService processManagerService(&processManager);
    QDBusConnection connection = QDBusConnection::sessionBus();
    if (!connection.registerService("com.kylin.ProcessManager") ||
        !connection.registerObject("/com/kylin/ProcessManager", &processManager)) {
        qWarning() << "Register process manager dbus service failed:" << connection.lastError();
    }

    return a.exec();
}
